/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {ExerciseCodeTemplateFullBase} from "../../../../types/exerciseEditor";
import {Table, Checkbox} from "semantic-ui-react";
import {CheckboxOnChangeEventData} from "../../../../types/reactEvents";
import {setCodeTemplates} from "../../../../state/actions/exerciseEditorSite/subSets/editorExerciseCodeTemplatesActions";


import {getI18n} from "../../../../../i18n/i18nRoot";
import {HelpPopup} from '../../../helpers/helpPopup'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly codeTemplate: ExerciseCodeTemplateFullBase
  readonly allCodeTemplates: ReadonlyArray<ExerciseCodeTemplateFullBase>
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setCodeTemplates
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class codeTemplateDetailsView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="pad-left-right">
        <Table padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId, 'File name')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span className="mar-right-half">

                  {
                    getI18n(this.props.langId, 'Content editable by user')
                  }
                </span>
                <HelpPopup defaultText={getI18n(this.props.langId,
                                                "Readonly files are always up-to-date with the version from the exercise. A copy of the readonly file is still stored in the users solution. If you change from normal to readonly file then the content from the user is not changed but no one can access the file content (backend will always return the up-to-date version from the exercise and not the user file content). If you change from readonly to normal file then the content is the same as the readonly file or if the file was changed by the user (because is was previously a normal file and changed to readonly prior and now back to normal) then the last state (content) is used when the user had saved the file before it got readonly."
                )} wide='very'/>
              </Table.HeaderCell>

              <Table.HeaderCell>
                <span className="mar-right-half">
                  {
                    getI18n(this.props.langId, "Content is visible for user")
                  }
                </span>
                <HelpPopup defaultText={getI18n(this.props.langId,
                                                "The file content is hidden from the user (empty file is displayed) and is readonly but the file name is visible. If this option is checked it overwrites the readonly option. If you change from hidden to visible to user will get the real file content. If you change from visible to hidden the user will get a blank file. The user file (with old content) will not get touched/updated only all backend calls will return the blank file. Also note that the user CAN get the file content when the program is executed (e.g. list all files in directory and read them)!"
                )} wide='very'/>
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              this.props.codeTemplate.templateFiles.map(file => {
                return (
                  <Table.Row key={file.id}>

                    <Table.Cell>
                      {
                        file.fileNameWithExtension
                      }
                    </Table.Cell>

                    <Table.Cell>
                      <div>
                        <Checkbox
                          disabled={file.isContentVisibleForUser === false}
                          checked={file.isEditableByUser}
                          onChange={(event: any, data: CheckboxOnChangeEventData) => {

                            this.props.setCodeTemplates(
                              this.props.allCodeTemplates.map(template => {
                                return template.id === this.props.codeTemplate.id
                                       ? {
                                    ...template,
                                    templateFiles: template.templateFiles.map(tempFile => {
                                      return tempFile.id === file.id
                                             ? {
                                          ...tempFile,
                                          isEditableByUser: data.checked
                                        }
                                             : tempFile
                                    })
                                  }
                                       : template
                              })
                            )
                          }}/>
                      </div>

                    </Table.Cell>

                    <Table.Cell>
                      <div>
                        <Checkbox checked={file.isContentVisibleForUser}
                                  onChange={(event: any, data: CheckboxOnChangeEventData) => {

                                    this.props.setCodeTemplates(
                                      this.props.allCodeTemplates.map(template => {
                                        return template.id === this.props.codeTemplate.id
                                               ? {
                                            ...template,
                                            templateFiles: template.templateFiles.map(tempFile => {
                                              return tempFile.id === file.id
                                                     ? {
                                                  ...tempFile,
                                                  isContentVisibleForUser: data.checked
                                                }
                                                     : tempFile
                                            })
                                          }
                                               : template
                                      })
                                    )
                                  }}/>
                      </div>

                    </Table.Cell>

                  </Table.Row>
                )
              })
            }
          </Table.Body>

        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(codeTemplateDetailsView)
