import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from '../../../state/reducers'
import {getI18n} from '../../../../i18n/i18nRoot'
import {loadSystemSettingsSite} from '../../../state/actions/systemSettingsSite/systemSettingsActions'
import {errorDialog} from '../../../helpers/dialogHelper'
import {
  getGlobalSystemSettingsAsync,
  getTestServerPublicSettingsAsync
} from '../../../state/actions/globalSystemSettings/globalSystemSettingsCrudActions'
import {ErrorHelper} from '../../../helpers/errorHelper'
import PlangSettingsView from './plangSettingsView'
import SettingsView from './settingsView'
import {getPLangsWithExtraDataAsync} from '../../../state/actions/plangSettingsView/pLangsCrudActions'
import LanguageSettingsView from './languageSettingsView'
import {getLangsAsync} from '../../../state/actions/langsView/langCrudActions'
import TestServerSettingsView from './testServerSettingsView'

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    systemSettings: rootState.systemSettingsSiteState.systemSettings,
    isLoading: rootState.systemSettingsSiteState.isLoading,

    systemRole: rootState.userDataSettingsSate.userData
                && rootState.userDataSettingsSate.userData.systemRole
                ? rootState.userDataSettingsSate.userData.systemRole
                : null,

    langId: rootState.i18nState.langId,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        loadSystemSettingsSite,
                                                                        getGlobalSystemSettingsAsync,
                                                                        getTestServerPublicSettingsAsync,
                                                                        getPLangsWithExtraDataAsync,
                                                                        getLangsAsync,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class SystemSettingsSite extends React.Component<Props, any> {


  async componentDidMount(): Promise<void> {

    if (this.props.systemRole === null || !this.props.systemRole.canChangeSystemSettings) {
      return
    }

    try {
      await this.props.loadSystemSettingsSite()
      await this.props.getPLangsWithExtraDataAsync()
      await this.props.getLangsAsync()
      await this.props.getGlobalSystemSettingsAsync() //this updates the server message when we just edited it
      await this.props.getTestServerPublicSettingsAsync()
    } catch (err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
    }

  }

  render(): JSX.Element {

    if (this.props.systemRole === null || !this.props.systemRole.canChangeSystemSettings) {

      errorDialog(getI18n(this.props.langId, "Error"), getI18n(this.props.langId, "No permission"),
                  this.props.langId)
      return null
    }

    return (
      <div className="multiple-single-panels">

        <SettingsView />

        <TestServerSettingsView />

        <PlangSettingsView />

        <LanguageSettingsView />

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(SystemSettingsSite)
