/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Icon} from "semantic-ui-react";
import CommonHeaderBarContentWrapper from '../../commonHeaderBarContentWrapper'


import {HelpPopup} from "../../helpers/helpPopup";
import {getI18n} from "../../../../i18n/i18nRoot";
import {
  changeSingleAssessmentTutorViewAsync,
  justRunProgramTutorViewAsync, runCompileSingleFileTutorView
} from '../../../state/actions/tutorViewSite/tutorViewSiteCrudActions'
import {SimpleVDivider} from '../../helpers/simpleVDivider'
import history from '../../../routerHistory'
import * as constants from '../../../constants'
import {paginationPageSizeOptionAll} from '../../../constants'
import {getTutorViewForSubmission} from '../../../constants'
import {Link} from "react-router-dom"
import {AssessmentFullBase} from '../../../types/submissions'
import {
  loadSubmissionsSite,
  resetSubmissionsSite, setActivePaginationPage, setPaginationPageSize, setSortByKey
} from '../../../state/actions/ownOrGroupExerciseActions/submissionsSite/submissionsSiteActions'
import {
  setIsLoadingTutorViewSite, setSelectedConsoleViewTest, setTutorViewIsJustRunProgramArgsInputDisplayed,
  setTutorViewJustRunProgramArgs
} from '../../../state/actions/tutorViewSite/tutorViewSiteActions'
import {ErrorHelper} from '../../../helpers/errorHelper'
import {
  JustRunProgramCommandForBackend, JustRunProgramTestTutorViewCommandForBackend,
  RunCompileSingleFileCommandForBackend, SolutionFileForTestServer
} from '../../../types/testsAndTestProtocol'
import {justRunProgramArgsMaxLength} from '../../../state/reducers/doExerciseSite/doExerciseSiteReducer'
import {ProgramArgsInputView} from '../../helpers/programArgsInputView'
import {SolutionFileDoExerciseFullBase} from '../../../types/exerciseSolution'
import {SimpleDivider} from '../../helpers/simpleDivider'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isLoading: rootState.tutorViewSiteState.isLoading,
    exerciseRelease: rootState.tutorViewSiteState.exercise.exerciseRelease,
    exercise: rootState.tutorViewSiteState.exercise,
    comesFromOwnExercisesSite: rootState.tutorViewSiteState.comesFromOwnExercisesSite,
    targetUserId: rootState.tutorViewSiteState.userSolution.userId,
    testTypes: rootState.testTypesState.testTypes,

    assessment: rootState.tutorViewSiteState.assessment,

    isSomeDialogDisplayed: rootState.tutorViewSiteState.isViewTestDialogDisplayed
      || rootState.tutorViewSiteState.isCodeEditorSettingsDialogDisplayed,

    isSomeTestRunning: rootState.tutorViewSiteState.exercise.tests.some(p => p.uiIsRunning)
      || rootState.tutorViewSiteState.exercise.customTests.some(p => p.uiIsRunning)
      || rootState.tutorViewSiteState.normalTestsForTutor.some(p => p.uiIsRunning)
      || rootState.tutorViewSiteState.submitTestsForTutor.some(p => p.uiIsRunning)
    ,

    releaseFinishedAndAutomaticAssessmentHasFinished: !rootState.tutorViewSiteState.exercise.exerciseRelease.isReleased &&
      rootState.tutorViewSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished,

    isTutorEditorSelected: rootState.tutorViewSiteState.topLeftSelectedTabIndex === 0,
    selectedTabIdTutorEditor: rootState.tutorViewSiteState.tutorEditorState.selectedTabIdDoEditor,

    tutorSolution: rootState.tutorViewSiteState.tutorEditorSolution,

    submissionsSiteRelease: rootState.submissionsSiteState.release,
    submissions: rootState.submissionsSiteState.submissions,
    submissionsSortByKey: rootState.submissionsSiteState.pagination.orderByProp,
    submissionsSortDirection: rootState.submissionsSiteState.pagination.isOrderByDesc,
    submissionsPagination: rootState.submissionsSiteState.pagination,

    justRunProgramArgs: rootState.tutorViewSiteState.justRunProgramArgs,
    isJustRunProgramArgsInputDisplayed: rootState.tutorViewSiteState.isJustRunProgramArgsInputDisplayed,

    shouldNotCount: rootState.tutorViewSiteState.userSolution.shouldNotCount,
    langId: rootState.i18nState.langId,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        changeSingleAssessmentTutorViewAsync,

                                                                        resetSubmissionsSite,
                                                                        loadSubmissionsSite,
                                                                        setIsLoadingTutorViewSite,

                                                                        setSortByKeySubmissionsSite: setSortByKey,
                                                                        setPaginationPageSizeSubmissionsSite: setPaginationPageSize,
                                                                        setActivePaginationPageSubmissionsSite: setActivePaginationPage,
                                                                        setTutorViewJustRunProgramArgs,
                                                                        setTutorViewIsJustRunProgramArgsInputDisplayed,
                                                                        justRunProgramTutorViewAsync,
                                                                        runCompileSingleFileTutorView,
                                                                        setSelectedConsoleViewTest,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class HeaderBar extends React.Component<Props, any> {
  render(): JSX.Element {


    //make sure the submissions site is in the initial loaded state ...
    //all submissions & no filters & no sorting ...
    const canSwitchToNextSubmission = this.props.submissions.length > 1
      && this.props.submissionsSortByKey.toLowerCase() === 'lastName'.toLowerCase()
      && this.props.submissionsSortDirection === false //ascending is default
      && this.props.submissionsPagination.searchText === ''
      && this.props.submissionsPagination.pageSize === paginationPageSizeOptionAll
      && this.props.submissionsPagination.totalItems === this.props.submissions.length
      && this.props.submissionsSiteRelease !== null //the submissions site release needs to be loaded
      && this.props.exerciseRelease.id === this.props.submissionsSiteRelease.id //the submissions site needs to have the same release


    const selectedUserIndex = this.props.submissions.findIndex(p => p.userId === this.props.targetUserId)
    const previousUserAssessment: AssessmentFullBase | null = (selectedUserIndex - 1) >= 0
                                                              ? this.props.submissions[(selectedUserIndex - 1)]
                                                              : null
    const nextUserAssessment: AssessmentFullBase | null = (selectedUserIndex + 1) < this.props.submissions.length
                                                          ? this.props.submissions[(selectedUserIndex + 1)]
                                                          : null

    const allFiles = this.props.tutorSolution.solutionFiles

    //this is not the real main file (saved in the solution) but the CURRENT FILE
    // noinspection TsLint
    const mainFile: SolutionFileDoExerciseFullBase | undefined = allFiles.find(
      p => p.id === this.props.selectedTabIdTutorEditor)


    const leftArea = (
      <div>

        <div className="flexed">

          {
            this.props.exercise.isPermanentlyLocked === false &&
            <div className="inline-flex">

              <div className={this.props.releaseFinishedAndAutomaticAssessmentHasFinished && this.props.assessment
                              ? 'clickable'
                              : 'div-disabled'}
                   onClick={() => {
                     //change = update = save
                     this.props.changeSingleAssessmentTutorViewAsync(this.props.assessment)
                   }}
              >
                <Icon name="save"/>
                <span>
                {
                  getI18n(this.props.langId, 'Save manual assessment')
                }
                </span>
              </div>

              <HelpPopup className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                        "This saves the manual assessment tab and the feedback for student tab"
              )}/>
            </div>
          }

          {
            (!this.props.releaseFinishedAndAutomaticAssessmentHasFinished || !this.props.assessment) &&
            <HelpPopup className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                      'The release is still open or the automatic assessment has not finished yet'
            )}/>
          }

          {
            this.props.exercise.isPermanentlyLocked === false || (!this.props.releaseFinishedAndAutomaticAssessmentHasFinished || !this.props.assessment) &&
            <SimpleVDivider/>
          }

          <Link className={canSwitchToNextSubmission && previousUserAssessment !== null
                           ? 'clickable mar-right-half'
                           : 'div-disabled mar-right-half'}
                to={getTutorViewForSubmission(this.props.comesFromOwnExercisesSite, this.props.exerciseRelease.id,
                                              previousUserAssessment !== null
                                              ? previousUserAssessment.userId
                                              : 0,
                                              previousUserAssessment !== null
                                              ? previousUserAssessment.pLangId
                                              : 0
                )}
          >
            <Icon name="arrow left"/>
            <span>
            {
              getI18n(this.props.langId, 'Previous')
            }
          </span>
          </Link>

          {
            canSwitchToNextSubmission &&
            <div>
              {
                selectedUserIndex !== -1
                ? `# ${(selectedUserIndex + 1)}/${this.props.submissions.length}`
                : getI18n(this.props.langId, "Error")
              }
            </div>
          }
          {
            !canSwitchToNextSubmission &&
            <div onClick={async () => {

              this.props.setIsLoadingTutorViewSite(true)
              this.props.resetSubmissionsSite()

              //this can produce an error because it cannot load the data when we resetted the release
              this.props.setSortByKeySubmissionsSite('lastName', false)
              this.props.setPaginationPageSizeSubmissionsSite(paginationPageSizeOptionAll)
              this.props.setActivePaginationPageSubmissionsSite(1)

              try {
                await this.props.loadSubmissionsSite(this.props.exercise.id, this.props.comesFromOwnExercisesSite,
                                                     this.props.exerciseRelease.id, false
                )
              } catch (err) {

                this.props.setIsLoadingTutorViewSite(false)
                ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error',
                                                  'Could not load submissions-list data'
                )
                return
              }


              this.props.setIsLoadingTutorViewSite(false)

            }}>
              <HelpPopup className="clickable" icon="exclamation triangle" wide defaultText={getI18n(this.props.langId,
                                                                               "The next/previous feature is only available if you came here from the submissions-list and submissions-list is sorted by 'lastName', all submission are loaded (page size 'all'), the search text is empty and there are more than 1 submission in total"
              )}
                         additionalText={getI18n(this.props.langId,
                                                 "Click here to set the right settings for the submissions-site and load the required data"
                         )}

              />
            </div>
          }

          <Link className={canSwitchToNextSubmission && nextUserAssessment !== null
                           ? 'clickable mar-left-half'
                           : 'div-disabled mar-left-half'}
                to={getTutorViewForSubmission(this.props.comesFromOwnExercisesSite, this.props.exerciseRelease.id,
                                              nextUserAssessment !== null
                                              ? nextUserAssessment.userId
                                              : 0,
                                              nextUserAssessment !== null
                                              ? nextUserAssessment.pLangId
                                              : 0
                )}
          >
            <span>
            {
              getI18n(this.props.langId, 'Next')
            }
            </span>
            <Icon name="arrow right" style={{marginLeft: '0.25rem'}}/>
          </Link>


          <SimpleVDivider/>

          {
            //compile single file
            !this.props.isLoading &&
            <div className="flexed">
              <div className={
                (allFiles.length > 0 && mainFile && this.props.isTutorEditorSelected && !this.props.isSomeTestRunning)
                ? 'clickable mar-right-half'
                : 'div-disabled mar-right-half'
              }
                   onClick={async () => {

                     if (allFiles.length === 0) {
                       return
                     }

                     if (!mainFile) {
                       //...maybe the exercise has no default files...
                       //or no file is selected...
                       return
                     }

                     const command: RunCompileSingleFileCommandForBackend = {
                       generatedCode: this.props.exerciseRelease.releaseCode,
                       plangId: this.props.tutorSolution.pLangId,
                       mainFileName: mainFile.fileNameWithExtension,
                       files: allFiles.map<SolutionFileForTestServer>(p => {
                         return {
                           fileName: p.fileNameWithExtension,
                           fileContent: p.content
                         }
                       })
                     }

                     //else the old result could be displayed...
                     //actually new test should be displayed (because in runCompileSingleFileDoExercise we forge a temp test)
                     //but this don't happen somehow so clear
                     this.props.setSelectedConsoleViewTest(null)

                     this.props.runCompileSingleFileTutorView(command, this.props.testTypes)

                   }}
              >
                <Icon name="bug"/>
                <span>
            {
              getI18n(this.props.langId, 'Compile single file')
            }
          </span>
              </div>
              <HelpPopup
                defaultText={getI18n(this.props.langId,
                                     'Compiles the program with the current file as main file.'
                )}
                wide
              />
            </div>
          }

          <SimpleVDivider/>

          {
            //just run program
            //behind compile single file in case we want to make cmd args accessible inline
            !this.props.isLoading &&
            <div className="flexed">
              <div className={
                (allFiles.length > 0 && this.props.isTutorEditorSelected && !this.props.isSomeTestRunning)
                ? 'clickable mar-right-half'
                : 'div-disabled mar-right-half'
              }
                   onClick={async () => {

                     if (allFiles.length === 0) {
                       return
                     }

                     const command: JustRunProgramTestTutorViewCommandForBackend = {
                       CmdArgs: this.props.justRunProgramArgs
                                ? this.props.justRunProgramArgs
                                : '',
                       releaseCode: this.props.exerciseRelease.releaseCode,
                       plangId: this.props.tutorSolution.pLangId,
                       mainFileName: mainFile.fileNameWithExtension,
                       files: allFiles.map<SolutionFileForTestServer>(p => {
                         return {
                           fileName: p.fileNameWithExtension,
                           fileContent: p.content
                         }
                       })
                     }

                     //clear old result
                     this.props.setSelectedConsoleViewTest(null)

                     this.props.justRunProgramTutorViewAsync(command)

                   }}
              >
                <Icon name="rocket"/>
                <span>
                {
                  getI18n(this.props.langId, 'Just Run Program')
                }
              </span>
              </div>

              <HelpPopup className="mar-right-half"
                         defaultText={getI18n(this.props.langId,
                                              'If one arg should contain whitespaces enclose it in "" because args are separated by whitespaces. The max length for the program args is '
                         ) + justRunProgramArgsMaxLength}
                         wide
              />

              <div style={{display: 'inline-block'}} className="clickable mar-right-half" onClick={() => {
                this.props.setTutorViewIsJustRunProgramArgsInputDisplayed(
                  !this.props.isJustRunProgramArgsInputDisplayed)
              }}>
                <Icon className={this.props.justRunProgramArgs
                                 ? 'toggled-colored'
                                 : ''}
                      name="font"/>
              </div>

              {
                this.props.isJustRunProgramArgsInputDisplayed &&
                <div>
                  <ProgramArgsInputView programArgs={this.props.justRunProgramArgs}
                                        setProgramArgs={this.props.setTutorViewJustRunProgramArgs}/>
                </div>
              }


            </div>
          }

          <SimpleVDivider/>

          <div className="clickable" onClick={() => {

            if (this.props.comesFromOwnExercisesSite === null) {
              //go back to assessment statistic
              history.push(
                constants.getAssessmentStatisticsSiteToLinkPath(this.props.targetUserId))
              return
            }

            history.push(
              constants.getSubmissionsSiteToLinkPath(this.props.exercise.id, this.props.comesFromOwnExercisesSite,
                                                     this.props.exerciseRelease.id
              ))
          }}>
            <Icon name="reply"/>
            <span>
                {getI18n(this.props.langId, 'Leave')}
              </span>
          </div>

        </div>


      </div>
    )

    const rightArea = (
      <div className="flexed">
        {
          this.props.shouldNotCount &&
          <div className="flexed">
            <Icon className="mar-right-half" name="flag outline"/>
            <span className="mar-right-half">
                {
                  getI18n(this.props.langId, 'Should not count')
                }
              </span>
            <HelpPopup
              defaultText={getI18n(this.props.langId, 'The assessment result/participation should not be counted.')}/>
          </div>
        }

        {
          this.props.exercise.isPermanentlyLocked &&
          <div className="inline-flex">
            <SimpleVDivider/>

            <div>
              <HelpPopup icon="lock" defaultText={getI18n(this.props.langId, 'Exercise is lock')}/>
            </div>
          </div>
        }

      </div>
    )

    return (
      <CommonHeaderBarContentWrapper leftArea={leftArea} rightArea={rightArea}
                                     disabled={this.props.isLoading || this.props.isSomeDialogDisplayed}/>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(HeaderBar)
