/**
 * Created by janis dähne.
 */



export interface AssessmentFullBase {

  /**
   * cant be added because the backend has only a join table for that --> composed id
   */
  //readonly id: number

  /**
   * the user token
   * not always sent, only when needed (e.g. export)
   * but e.g. not for tutor view
   */
  readonly token: string | null

  /**
   * the first name of the connected user
   */
  readonly firstName: string
  /**
   * the last name of the connected user
   */
  readonly lastName: string

  /**
   * this is taken from the users solution
   * true: the student wants that his/her solution is not counted (for whatever), false: not
   * can only be set once by the student
   */
  readonly shouldNotCount: boolean

  /**
   * the user id (in case the tutor wants to see the solution NOT SUPPORTED YET)
   */
  readonly userId: number
  /**
   * the used plang
   */
  readonly pLangId: number

  /**
   * the connected exercise release id
   */
  readonly exerciseReleaseId: number

  /**
   * the summed test points from the submit tests
   */
  readonly submitTestPoints: number | null

  /**
   * the summed max reachable submit test points
   * this is even set for dummy assessments
   */
  readonly maxSubmitTestPoints: number

  /**
   * the summed points from the normal test
   */
  readonly normalTestPoints: number | null

  /**
   * the summed max reachable normal test points
   * this is even set for dummy assessments
   */
  readonly maxNormalTestPoints: number

  /**
   * optional the manual given points (by the tutor)
   */
  readonly manualPoints: number | null

  /**
   * a hint for the max manual points, manual points can be greater than this value
   */
  readonly maxManualPoints: number | null

  /**
   * the note from the tutor
   */
  readonly noteForOtherTutors: string | null

  /**
   * feedback for the student
   */
  readonly feedbackForStudent: string | null

  /**
   * a user can solve an exercise multiple times (connected through release.exercise)
   * so this gives us the times the user participated on the exercise (through different releases)
   *
   * IF a user solved the exercise with different p langs then this does not count separate because
   * all restrictions are stored at the participation (e.g. time) and thus the p lang does not matter!
   *
   * CAN be -1 on error or on unset (heavy to compute...)
   */
  readonly participationCount: number

  /**
   * null or empty string for no error OR
   * the error message from the test server or some other source
   */
  readonly lastAssessmentErrorMessage: string | null

  /**
   * the last ip address when a user created or changed the solution
   * can be null if not captured (before we introduced this prop)
   */
  readonly lastEditingIpAddress: string | null
}
